#! /usr/bin/perl 
# vim:ts=4
# You cant have -w since HTTP::Cookie throws a warning if you get a null
# cookie
#
# Version 0.2
#
# Check a website user experience: follow links etc preserving
# cookies and parameters (if given).
#
# Syntax:  check_website [-M|-N [-w warn -c crit]] -s <scriptfile> 
#                        [-d][-h] [-D symbol=value ... ]
#
# scriptfile contains lots of stuff.  See example for details.

#use strict; # this comes later
use Text::ParseWords;
use Time::HiRes qw( usleep ualarm gettimeofday tv_interval );
use LWP::UserAgent;
use HTTP::Cookies;
use Getopt::Long;

my( $VERSION ) = "0.2";

my( $MRTG ) = 0; # MRTG or Nagios mode
my( $TIME ) = 0;
my( $WARN, $CRIT ) = ( 1000, 2000 );
my( $MESSAGE ) = "";
my( $STATUS ) = 3;
my( $CONFIG ) = "script.scr";
my( $TIMEOUT ) = 10; #seconds
my( $COOKIES );
my( $USERNAME, $PASSWORD ) = ('','');
my( $MAXLOOP ) = 1;
my( $LOGFILE ) = "/tmp/check.log";
my( $DEBUG ) = 0;
my( $DEADTIME ) = 0;

use vars qw/@opt_D $opt_s $opt_h $opt_d $opt_M $opt_N $opt_w $opt_c/;
my( $config );
my( $ua );
my( $res, $req ) = ('','');
my( $k, $v);
my( %symbols ) = ();
my( @ALL ) = ();
my( $postit, $curline, $depth , $options, $starttime, $url, $rv, $param);
my( %lastform, $isform, $loopcnt );
####################################################################
# LWP::AutoUAL
# User Agent with overrides for the authentication functions
package LWP::AutoUA;
require LWP::UserAgent;
@ISA = qw(LWP::UserAgent);
sub get_basic_credentials(@) {
	return (undef,undef) if(!$USERNAME);
	return ($USERNAME,$PASSWORD);
}
package main;
use strict;

####################################################################
# Object for config list.
####################################################################
sub islist($) { return 1 if((ref $_[0]) eq 'LIST'); return 0; }
package LIST;
sub list { return LIST->new(@_); }
sub new {
    my ($me) = shift;
    my ($name) = shift;
    my (@initial) = @_;
    my ($list) = {};
    $name = "" if(!defined $name);
    $$list{name} = $name;
    $$list{items} = [ @initial ];
    bless $list, "LIST";
    return $list;
}
sub name { my ($me) = shift; return $$me{name}; }
sub items { my ($me) = shift; return @{$$me{items}}; }
sub add {
    my ($me) = shift;
    my (@items) = @_;
    push @{$$me{items}},@items;
}
sub setlist {
    my ($me) = shift;
    my (@items) = @_;
    $$me{items} = [ @items ];
}
sub set {
    my ($me) = shift;
    my ($item) = shift;
    my (@items) = @_;
    return 1 if(ref $me ne 'LIST');
    foreach ( @{$$me{items}} ) {
        if( ref $_ eq 'LIST' ) {
            if( lc $_->name eq lc $item ) {
                $_->setlist(@items);
                return 0;
            }
        } else {
            if( lc $_ eq lc $item ) { return 1; } # exists, not as list
        }
    }
    # doesnt exist yet, so create it
    push @{$$me{items}},LIST->new($item,@items);
    return 0;
}
sub addlist {
    my ($me) = shift;
    my ($name) = shift;
    my (@items) = @_;
    push @{$$me{items}},LIST->new($name,@items);
}
sub item {
    my ($me) = shift;
    my ($item) = shift;
    return '' if(!$item);
    foreach ( @{$$me{items}} ) {
        if( ref $_ eq 'LIST' ) { return $_ if( lc $$_{name} eq lc $item ); }
        else { return $_ if( lc $_ eq lc $item ); }
    }
    return '';
}
# like item, but returns first element of single-item list
sub opt {
    my( $me ) = shift;
    my( $item ) = shift;
    my( @l ) = ();
    my( $i );
    return '' if(!$item);
    foreach $i ( @{$$me{items}} ) {
        if( ref $i eq 'LIST' ) {
            if( lc $$i{name} eq lc $item ) {
                @l = @{$$i{items}};
                return '' if(!@l);
                return $l[0] if(defined $l[0] and ((ref $l[0] ne 'LIST')
                    or($l[0]->{name} ne 'LINE')));
                return $l[1] if(defined $l[1]);
                return '';
            }
        } else { return $_ if( lc $i eq lc $item ); }
    }
    return '';
}
sub opts {
    my( $me ) = shift;
    my( $item ) = shift;
    my( @l ) = ();
    my(@rv ) = ();
    my( $i );
    return @rv if(!$item);
    foreach $i ( @{$$me{items}} ) {
        if( ref $i eq 'LIST' ) {
            if( lc $$i{name} eq lc $item ) {
                foreach ( @{$$i{items}} ) {
                    if( ref $_ eq 'LIST' ) {
                        push @rv,$_->name if( $_->name ne 'LINE' );
                    } else { push @rv, $_; }
                }
                last;
            }
        } else { last if( lc $i eq lc $item ); }
    }
    return @rv;
}
sub member {
    my ($me) = shift;
    my ($item) = shift;
    return 0 if(!$item);
    foreach ( @{$$me{items}} ) {
        if( ref $_ eq 'LIST' ) { return 1 if( lc $$_{name} eq lc $item ); }
        else { return 1 if( lc $_ eq lc $item ); }
    }
    return 0;
}
####################################################################
package main;

sub expand($) {
	my( $str ) = $_[0];
	my($k,$v);
	# Parameters to globally replace
	while( $str =~ /\$(\w+)\$/ ) {
		$k = $1; $v = '';
		if(defined $symbols{$k}) {
			$v = $symbols{$k} ;
		} else {
			$v = 'Undefined';
		}
		$str =~ s/\$$k\$/$v/g;
	}
	return $str;
}

sub checkconfig {
    # Check the syntax of the parsed config file
    my($sl,$l,$n,$sn);
    my($m,$a,$g);
    my($rv) = 0;
    # Check: first level labels are appropriate
	$curline = 1;
    foreach $l ( $config->items ) {
        if(ref($l) ne 'LIST') {
            print "Warning: Bareword '$l' found near line $curline\n";
            $rv = 1;
            next;
        }
        $curline = $l->opt('LINE');
        if($l->name eq 'LINE') { next; }
        $n = $l->name;
        if( $n ne 'option' and $n ne 'fetch' and $n ne 'sleep' ) {
            print "ERROR: top-level label '$n' not recognised at line $curline\n";
            $rv += 1;
            next;
        }
        # Check: second level labels are appropriate
        $m = $g = $a = 0;
        foreach $sl ( $l->items ) {
            if(ref($sl) ne 'LIST') {
				if( !$m and $sl and !($l->member('url')) ) {
					$m = 1;	
					$l->set('url',$sl);
					next;
				}
				if( $sl and $l->name eq 'sleep' ) {
					next;
				}
                print "ERROR: Bareword '$sl' found at line $curline\n";
                $rv += 1;
                next;
            }
            $curline = $sl->opt('LINE');
            $sn = $sl->name;
            if($sn eq 'LINE') { next; }

            if( $n eq 'option' ) {
                next if( $sn eq 'timeout' or $sn eq 'loop' 
					or $sn eq 'keepalive' or $sn eq 'param'
					or $sn eq 'warning' or $sn eq 'critical'
					or $sn eq 'username' or $sn eq 'password' );
                print "WARNING: unknown keyword '$sn' defined in options definition at line $curline\n";
                $rv++; next;
            }
            next if( $sn eq 'url' or $sn eq 'param' or $sn eq 'parse' 
				or $sn eq 'expect' or $sn eq 'form' or $sn eq 'notexpect' );

            print "WARNING: unknown keyword '$sn' defined in $n definition at line $curline\n";
            $rv++; next;
        }
        $rv = 1 if($m>1 or $a>1 or $g>1);
    }
    return $rv;
}

sub clean {
    my(@rv);
    foreach ( @_ ) { push @rv,$_ if(!islist($_)); }
    return @rv;
}

# read list from @ALL (recursive!)
sub readlist($) {
    my($lname) = $_[0];
    my($l, $tok);

    $l = LIST->new($lname); # create new list by this name
    $l->addlist('LINE',$curline);
	$depth = 0 if(!$depth);
    $depth = $depth + 1;

    while( @ALL ) {
        do { last if(!@ALL); $tok = shift @ALL; }
            while(!defined $tok or $tok =~ /^\s*$/ );
        if( $tok =~ /^LINE:(\d+)/ ) { $curline = $1; next; }
		$tok =~ s/ \( /\(/g; $tok =~ s/ \) /\)/g; # detoken brackets
#        print "{$tok}\n" if($debug);
        # now, is this an item or a new list?
        if( $tok eq '(' ) {
            print "WARNING: Line $curline: Unexpected '(' at start of $lname definition.\n"; next;
        }
        if( $tok eq ')' ) { $depth = $depth - 1; return $l; } # End of list!
        if( $ALL[0] and $ALL[0] eq '(' ) {
            # new list
            shift @ALL;
            $tok = lc $tok; # force to lower, chop trailing s
            $tok =~ s/(option|param)s/$1/;
            $l->add( &readlist($tok) );
        } else { $l->add($tok); } # item
    }
    # if we reach here, the config is exhausted and there is an unbalanced (
    print "ERROR: Line $curline: Unexpected end of file: unbalanced ( in definition of $lname.\n"
        if($depth > 1); # dont need final closing.
    return $l;
}
# Parse the config file
sub readcfg {
    my(@cur,$line, @line,$lnum);

    # Suck in the whole file
    open CFG, "<$CONFIG" or do {
        print "Unable to open $CONFIG!\n";
        exit 1;
    };
    $lnum = 0;
    while( <CFG> ) {
        $lnum = $lnum + 1;
        chomp; $line = $_;
        next if( $line =~ /^\s*#/ ); # skip comments
        next if( $line =~ /^\s*$/ ); # skip blank
        $line =~ s/([\(\)])/ $1 /g; # make sure brackets are separate tokens
#       print "[ $line ]\n" if($debug);
        @line =  quotewords( '[\s,]+', 0, $line );
        push @ALL, "LINE:$lnum"; # for benefit of error messages
        push @ALL, @line;
    }
    close CFG;
    # now process it
    $config = readlist("TOP");
}

######################################################################
# Fetching

my($LASTHEAD) = '';
my($LASTPATH) = '';
my($LASTURL) = '';

sub fetchurl($$) {
	my($u,$p) = @_;

	if($u =~ /^\//) {
		$u = $LASTHEAD.$u;
	} elsif( $u =~ /^\?/ ) {
		$u = $LASTURL.$u;
	} elsif( $u !~ /^(https?|ftp)/ ) {
		$u = $LASTHEAD.$LASTPATH.$u;
	}

	if($DEBUG) {
		open LOG, ">>$LOGFILE";
		print LOG "---------------------------------\nNEW REQUEST\n"
			."URL: $u\nPARAM: $p\nLAST: $LASTURL\n"
			."-----------------------------------\n";
		close LOG;
	}
	
         # Create a request
	if($p) {
        $req = HTTP::Request->new(POST=>$u);
		$req->content_type("application/x-www-form-urlencoded");
	# set content?
		$req->content($p);
	} else {
		$req = HTTP::Request->new(GET=>$u);
	}

	$req->headers->{'Referer'} = $LASTURL if($LASTURL);

    # Pass request to the user agent and get a response back
	$u =~ s/ /+/g;
	$COOKIES->add_cookie_header($req);
	if($DEBUG) {
		print "--> Add Cookie: ".$req->headers->{cookie}."\n" 
			if(defined $req->headers->{cookie} );
	}
	print "--> Retrieving $u ".($p?"($p)":"")."\n" if($DEBUG);
	$res = $ua->request($req);
	print "--> Done.\n" if($DEBUG);

	if(!$res) { return "Error: unable to retrieve"; }

    # Check the outcome of the response
	if( $res->code < 500 ) {
		if($DEBUG and defined $res->headers->{'set-cookie'}) { 
			if ( ref $res->headers->{'set-cookie'} ) { 
				foreach ( @{$res->headers->{'set-cookie'}} ) { 
					print "--> Recieve Cookie: $_\n" ; 	
					if($DEBUG) { 
						open LOG, ">>$LOGFILE"; 
						print LOG "Cookie: $_\n"; 
						close LOG; 
					} 
				} 
			} elsif ( $res->headers->{'set-cookie'} ) { 
				print "--> Recieve Cookie: " 
					.$res->headers->{'set-cookie'}."\n" ; 
				if($DEBUG) { 
					open LOG, ">>$LOGFILE";
					print LOG "Cookie: ".$res->headers->{'set-cookie'}."\n";
					close LOG;
				}
			}
		}
		$COOKIES->extract_cookies($res);
		if($u =~ /^(.*\/\/[^\/]+)(.*\/)([^\?]*)/ ) {
			$LASTHEAD = $1; $LASTPATH = $2;
			$LASTURL = "$1$2$3";
		}
	}
    if ($res->is_success) {
        return 0;
    } else {
        return "Error: ".$res->message;
    }

}

sub getformurl($) {
	my($formurl) = '';
	my($post) = 0;
	my($res) = $_[0];
	return '' if(!$res);
	if( $res->content =~ /<FORM [^>]*(POST)?[^>]*action=([^\s>]+)[^pP>]*(POST)?/i ) {
		$formurl = $2;
		$post = 1 if($1 or $3);
	}
	$formurl =~ s/"//g;
	print "--> Parsed form URL [$formurl] (".($post?"POST":"GET").")\n" 
		if($DEBUG);
	return ($formurl,$post);
}

sub parsehtml($) {
	my($html) = $_[0];
	my($tag,@tag,$t,%tag);
	my($n,$v,$sn,$k);

	$sn = ''; 
	while( $html =~ s/<([^>]+)>// ) {
		$tag = $1;  $tag =~ s/ = /=/g;
		@tag = quotewords( '[\s,]+', 0, $tag );
		next if(!@tag);
		next if($tag[0]=~ /^\//);
		%tag = ();
		foreach ( @tag ) { 
			next if(!$_);
			if( /"([^"]+)"\s*=?\s*"?([^"]*)/ ) 
				{ $k = lc $1; $tag{$k} = $2; next; }
			if( /([^\s=]+)\s*=?\s*"?([^"]*)/ ) 
				{ $k = lc $1; $tag{$k} = $2; next }
		}
		$n = $tag{name}; $v = $tag{value}; 	
		$v = '' if(!defined $v); $n = '' if(!defined $n);
		$v =~ s/"//g;
		$v =~ s/=/%3d/g; $v =~ s/&/%26/g; $n =~ s/=/%3d/g; $n =~ s/&/%26/g;
		if( defined $tag{form} and $tag{action} ) { 
			$symbols{"FORM:$n"} = $tag{action}; 
			%lastform = ();
			print "--> Parsed FORM tag $n=".$tag{action}."\n" if($DEBUG);
		} elsif( defined $tag{input} ) { 
			if( !defined $tag{type}
				or $tag{type} =~ /hidden|pass|text/i 
				or defined $tag{checked} ) {
			$symbols{$n} = $v if($n); 
			$lastform{$n} = $v if($n); 
			print "--> Parsed INPUT tag $n=$v\n" if($DEBUG);
			}
		} elsif( defined $tag{'select'} ) { $sn = $n; }
		elsif( defined $tag{option} and defined $tag{selected} ) {
			$symbols{$sn} = $v if($sn); 
			$lastform{$sn} = $v if($sn); 
			print "--> Parsed SELECT tag $sn=$v\n" if($DEBUG);
		}
	}
}

sub dosleep($) {
	my($sec) = $_[0];

	print "Sleeping for $sec seconds.\n" if($DEBUG);
	$DEADTIME += $sec * 1000;
	$TIMEOUT += $sec;
	sleep $sec;
	
}

sub dohelp() {
	print "Usage: check_website [-N [-w <warnsec> -c <critsec>]|-M] -s <script>\n";
	print "-d : debug, -M MRTG mode, -N Nagios mode, -w/-c warn/crit seconds\n";
	exit 3;
}

########################################################################
# MAIN

# process arguments: at least MRTG/Nagios mode, and warn/crit for Nagios
#getopts('hdMNw:c:s:D:');
$Getopt::Long::autoabbrev = 0;
$Getopt::Long::ignorecase = 0;
GetOptions('h|help','d|debug','M|mrtg|MRTG','N|Nagios|nagios',
	'w|warn|warning=i','c|crit|critical=i','s|script=s','D|define=s@');
dohelp() if($opt_h);
$DEBUG=1 if($opt_d);
$MRTG = 0 if($opt_N);
$MRTG = 1 if($opt_M);
$WARN = $opt_w*1000 if($opt_w);
$CRIT = $opt_c*1000 if($opt_c);
$CONFIG = $ARGV[0] if($ARGV[0]);
$CONFIG = $opt_s if($opt_s);
foreach ( @opt_D ) {
	($k,$v) = split /=/,$_;
	next if(!$k);
	$symbols{$k} = $v;
	print "==> Added initial symbol $k=$v\n" if($DEBUG);
}

# read in config file
readcfg;
checkconfig;
$options = $config->item('option');

# Create and set up the useragent
$ua = LWP::AutoUA->new;
$ua->agent("CheshireCheck/1.0 ");
if($options) {
	$PASSWORD = $options->opt('password') 
		if($options->member('password'));
	$USERNAME = $options->opt('username') 
		if($options->member('username'));
	$TIMEOUT = $options->opt('timeout') 
		if($options->member('timeout'));
	$WARN = $options->opt('warning') * 1000
		if($options->member('warning') and !$opt_w);
	$CRIT = $options->opt('critical') * 1000
		if($options->member('critical') and !$opt_c);
	$MAXLOOP = $options->opt('loop') 
		if($options->member('loop'));
	if($options->member('keepalive')) {
		$ua->{keep_alive} = 1 ;
		print "==> Option: keep_alive\n" if($DEBUG);
	}
	if( $options->member('param') ) {
		foreach my $opt ( $options->items ) {
			next if(!islist($opt));
			next if($opt->name ne 'param');
			$k = ''; $v = '';
			foreach ( $opt->items ) {
				next if(islist($_)) ;
				if(!$k) { $k=$_; next; }
				$v = $_;
			}
			$v =~ s/^"//; $v =~ s/"$//;
			$symbols{$k} = $v;
			print "--> (Added initial symbol $k=$v)\n" if($DEBUG);
		}	
		$isform = 1;
	}
}
$ua->timeout($TIMEOUT) ;
push @{ $ua->requests_redirectable }, "POST";
$COOKIES = HTTP::Cookies->new;
$ua->cookie_jar($COOKIES);
# do something about username and password

# Store time 
$starttime = [gettimeofday];
$TIME = 0;

# work through things
foreach my $action ( $config->items ) {
	next if(!islist($action)); # so the next doesnt fail
	if( $action->name eq 'sleep') {
		my @l = $action->items;
		while( @l and islist($l[0]) ) { shift @l; }
		dosleep($l[0]) if(@l);
		next;
	}
	next if( $action->name ne 'fetch');
	$postit = 1;


	# find the URL
	$url = ''; $isform = 0;
	if( $action->member('form') ) {
		parsehtml($res->content) 
			if(! defined $symbols{"FORM:".$action->opt('form')} and $res);
		$url = $symbols{"FORM:".$action->opt('form')};
		print "Using form [".$action->opt('form')."] query URL\n" 
			if($url and $DEBUG);
		$isform = 1 if($url);
	}
	$url = $action->opt('url') if($action->member('url'));
	if(!$url) {
		# try last known form
		($url,$postit) = getformurl($res) if($res);
		print "Using form submit URL [$url]\n" if($url and $DEBUG);
		$isform = 1 if($url);
	} else {
		print "Fetching URL [$url]\n" if($DEBUG);
	}
	if(!$url) {
		print "No URL specified for fetch at line ".$action->opt('LINE')."\n";
		next;
	}
	
	# Parameters to globally replace
	$url = expand($url);
	print "--> (Becomes [$url])\n" if($DEBUG);

	# any params to pass
	$param = "";
	if( $action->member('param') ) {
		foreach my $opt ( $action->items ) {
			next if(!islist($opt));
			next if($opt->name ne 'param');
			$k = ''; $v = undef;
			foreach ( $opt->items ) {
				next if(islist($_)) ;
				if(!$k) { $k=$_; next; }
				$v = $_;
			}
			$v = $symbols{$k} if(!defined $v);
			$v = '' if(!defined $v);
			$v =~ s/^"//; $v =~ s/"$//;
			$lastform{$k} = $v;
		}	
		$isform = 1;
	}
	if( $isform ) {
		print "--> Processing parameters for form\n" if($DEBUG);
		foreach ( keys %lastform ) {
			($k, $v) = ( $_, $lastform{$_} );
			$param .= '&' if($param);
			$param .= "$k=".expand($v);
			print "--> (Added parameter $k=".expand($v).")\n" if($DEBUG);
		}
	}

	if(! $postit and $param ) {
		print "--> Using GET instead of POST for parameters\n" if($DEBUG);
		if($url =~ /\?/) {
			$url .= "&$param";
		} else {
			$url .= "?$param";
		}
		$param = '';
	} 

	# Fetch URL : if fail, set TIME=-1 and MESSAGE
	$rv = 'Not Fetched'; $loopcnt = 0;
	while( $rv ) {
		$rv = fetchurl($url,$param);
		if( $rv and $res and $res->headers->{location} ) {
			print "--> Redirecting again....\n" if($DEBUG);
			$url = $res->headers->{location}; $param = '';
			if($DEBUG) {
				open LOG, ">>$LOGFILE";
				print LOG "Resp: $rv\nRedirect to: $url\n";
				close LOG;
			}
		}
		$loopcnt += 1;
		last if($loopcnt >= $MAXLOOP);
	}
	if($rv) {
		$TIME = -1;
		$MESSAGE = "$rv";
		if($DEBUG) {
			print "==> FETCH FAILED $rv\n";
		}
		last;
	}
	$TIME = tv_interval($starttime);
	print "==> Retrieved OK!\n" if($DEBUG);
	if($DEBUG) {
		open LOG, ">>$LOGFILE";
		print LOG $res->content."\n---------------------------------\n";
		close LOG;
	}
	if( $TIME > $TIMEOUT ) {
		$TIME = -1; $MESSAGE = "Timeout at $TIMEOUT seconds";
		last;
	} else { 
		print "Time so far $TIME sec (timeout at $TIMEOUT)\n" if($DEBUG);
	}
 	$TIME *= 1000; # time in millisec

	# Find any response parsing to do
	if( $action->member('parse') ) {
		my($gotone) = 0;
		foreach $k ( $action->opts('parse') ) {
			$gotone = 1;
			if( $res->content =~ /name\s*=\s*"?$k[^>]+value\s*=\s*"?([^\s">]+)/i ) {
				$symbols{$k}=$1;
				print "--> Parsed symbol [$k=$1]\n" if($DEBUG);
			} elsif( $res->content =~ /value\s*=\s*"?([^\s">]+)[^>]+name\s*=\s*"?$k/i ) {
				$symbols{$k}=$1;
				print "--> Parsed symbol [$k=$1]\n" if($DEBUG);
			} else {
				print "--> Failed to parse symbol [$k]\n" if($DEBUG);
			}
		}
		if(!$gotone) { parsehtml($res->content); }
	}
	if( $action->member('expect') ) {
		my $breakout = 0;
		foreach $k ( $action->opts('expect') ) {
			if( $res->content !~ /$k/ ) {
				print "==> Couldn't find expected string [$k]\n" if($DEBUG);
				$TIME = -1; $MESSAGE = "Expected '$k' not found in response!";
				$breakout = 1; last;
			} else {
				print "--> Found expected string [$k]\n" if($DEBUG);
			}
		}
		last if($breakout);
	}
	if( $action->member('notexpect') ) {
		my $breakout = 0;
		foreach $k ( $action->opts('notexpect') ) {
			if( $res->content =~ /$k/ ) {
				print "==> Found unexpected string [$k]\n" if($DEBUG);
				$TIME = -1; $MESSAGE = "Unexpected '$k' found in response!";
				$breakout = 1; last;
			} else {
				print "--> Didnt find unexpected string [$k]\n" if($DEBUG);
			}
		}
		last if($breakout);
	}
}

$TIME -= $DEADTIME;

print "Whole process took $TIME msec\n" if($DEBUG and ($TIME>0));
print "-----------------------------------------------------------------\n"
	if($DEBUG);

# exit
if($MRTG) {
	if($TIME<1) {
		$TIME = 'U';
		$MESSAGE = 'Unable to complete script' if(!$MESSAGE);
	} else {
		$MESSAGE = "Process completed in $TIME msec" if(!$MESSAGE);
	}
	print "$TIME\n$TIME\n\n$MESSAGE\n";
	exit 0;
} else {
	if($TIME < 1) {
		$MESSAGE = 'Unable to complete script' if(!$MESSAGE);
		$STATUS = 2 if(!$STATUS);
	} elsif( $TIME > $CRIT ) {
		$MESSAGE = "CRITICAL: Process completed in $TIME msec" if(!$MESSAGE);
		$STATUS = 2;
	} elsif( $TIME > $WARN ) {
		$MESSAGE = "WARNING: Process completed in $TIME msec" if(!$MESSAGE);
		$STATUS = 1;
	} else {
		$MESSAGE = "Process completed in $TIME msec" if(!$MESSAGE);
		$STATUS = 0;
	}
	print "$MESSAGE\n";
	exit $STATUS;
}
# NOT REACHED
exit -1;
